@@ -291,7 +291,8 @@ def aggregate_result_file(job_desc_file, output_dir, aggregate_output_dir, has_c
             left_on=left_join_cols,
             right_on=right_join_cols,
             how='left').drop(labels=right_join_cols, axis=1)    
-
+        # remove duplicate rows
+        final_peptide_table = final_peptide_table.drop_duplicates()
         merged_results['peptide_table'] = final_peptide_table
 
     # Create Pivot Table from peptide_table
@@ -303,7 +304,6 @@ def aggregate_result_file(job_desc_file, output_dir, aggregate_output_dir, has_c
         # binding.median_percentile in the merged_results['peptide_table'] is the median percenile by allele for each peptide
         # create pivot table on peptide that inludes the median percentiles by allele
 
-        print(merged_results['peptide_table'].to_csv())
         # Pivot the original DataFrame to spread method-allele combinations as columns
         pivot_df =  merged_results['peptide_table'].pivot_table(index='core.peptide', 
                                 columns='core.allele', 
@@ -330,7 +330,8 @@ def aggregate_result_file(job_desc_file, output_dir, aggregate_output_dir, has_c
         # remove 'binding.consensus_percentile_rank' and 'binding.consensus_adjusted_percentile' from percentile_columns
         percentile_columns = [col for col in percentile_columns if col not in ['binding.consensus_percentile_rank', 'binding.consensus_adjusted_percentile']]
         # create table with all percentiles
-        all_percentile_pivot_df = merged_results['peptide_table'].pivot(index='core.peptide', columns='core.allele', values=percentile_columns)
+        # Use multiple columns as the index to avoid duplicate peptides in different sequences/positions
+        all_percentile_pivot_df = merged_results['peptide_table'].pivot(index=['core.sequence_number', 'core.peptide', 'core.start'], columns='core.allele', values=percentile_columns)
 
         # Flatten the MultiIndex columns to have unique names
         all_percentile_pivot_df.columns = ['.'.join(col).strip() for col in all_percentile_pivot_df.columns.values]
@@ -347,6 +348,7 @@ def aggregate_result_file(job_desc_file, output_dir, aggregate_output_dir, has_c
         merged_results['peptide_table'] = pd.merge(merged_results['peptide_table'], subset_pivot_df, on='core.peptide', how='left')
 
         # Print result
+        #print("final_peptide_table")
         #print(final_peptide_table)
         # print(pivot_df.to_csv())
         merged_results['consolidated_peptide_table'] = pivot_df
