@@ -7,11 +7,11 @@ import sys
 import math
 
 from sequences import Proteins
-from allele_validator import Allele_Validator
+from allele_validator import AlleleValidator
 # from nxg-tools package
 from nxg_common.nxg_common import save_file_from_URI
 
-MHCI_CONSENSUS_METHOD_SET = ('smm', 'ann', 'comblib_sidney2008')
+MHCII_CONSENSUS_METHOD_SET = ('comblib', 'smm_align', 'nn_align', 'tepitope')
 
 def read_json_file(file_path):
     with open(file_path, 'r') as r_file:
@@ -24,14 +24,16 @@ def save_json(result, output_path):
     with open(output_path, 'w') as w_file:
         json.dump(result, w_file, indent=2)
 
-def get_peptide_list_with_start_and_stop_and_sequencenumber(input_sequence_text, alleles, peptide_length_range='asis'):
+def get_peptide_list_with_start_and_stop_and_sequencenumber(input_sequence_text, alleles, peptide_length_range='asis', peptide_spacing=1):
     table_data = []
     sequence_list = get_sequence_list(input_sequence_text)
     if not peptide_length_range or peptide_length_range == 'asis':
         # means lengths = ['asis',]
         lengths = ['asis',]
-    else:
+    elif peptide_spacing:
         lengths = range(peptide_length_range[0], peptide_length_range[1]+1)
+    else:
+        raise ValueError('if peptide_length_range not asis, peptide_shift (peptide_spacing) must be given')
     for allele in alleles:
         peptide_index = 1
         for seq_num, sequence in enumerate(sequence_list) :
@@ -46,7 +48,7 @@ def get_peptide_list_with_start_and_stop_and_sequencenumber(input_sequence_text,
                     peptide_index += 1
                 else:
                     length = int(length)
-                    for i in range(len(seq)-length+1):
+                    for i in range(0, len(seq)-length+1, peptide_spacing):
                         peptide = seq[i:i+length]
                         start = i+1
                         end = i+length
@@ -93,8 +95,10 @@ def split_peptides_with_diff_length(input_sequence_text, peptide_length_range):
         return peptides_with_diff_length
 
 
+# this function will split the peptides into batches, by length as well as by batch size
 def split_peptides_with_diff_length_batch(input_sequence_text,
                                           peptide_length_range,
+                                          peptide_spacing = 1,
                                           batch_size=8192,
                                           max_batches=32):
     """_summary_
@@ -108,7 +112,7 @@ def split_peptides_with_diff_length_batch(input_sequence_text,
     Returns:
         _type_: _description_
     """
-
+    #print('splitting peptides with different lengths and batching')
     sequence_list = get_sequence_list(input_sequence_text)
     if not peptide_length_range:
         # means lengths = ['asis',]
@@ -128,7 +132,7 @@ def split_peptides_with_diff_length_batch(input_sequence_text,
             for sequence in sequence_list:
                 seq = sequence['sequence']
                 length = int(length)
-                for i in range(len(seq)-length+1):
+                for i in range(0, len(seq)-length+1, peptide_spacing):
                     peptide = seq[i:i+length]
                     peptide_list.append(peptide)
             
@@ -143,13 +147,13 @@ def split_peptides_with_diff_length_batch(input_sequence_text,
         # now we go through and divide the peptide lists into batches, keeping
         # peptides of the same length together            
         
-        print(f'total peptides: {total_peptides}')
-        print(f'num lengths {num_lengths}')
+        #print(f'total peptides: {total_peptides}')
+        #print(f'num lengths {num_lengths}')
         
         # this is the maximum number of peptides that will fit
         # into the max_batches with the given batch size
         max_batch_peptide_limit = max_batches * batch_size
-        print(f'max batch peptide limit: {max_batch_peptide_limit}')
+        #print(f'max batch peptide limit: {max_batch_peptide_limit}')
         
         # since we need to also batch by length, we have to determine
         # the maximum number of batches per length
@@ -159,7 +163,7 @@ def split_peptides_with_diff_length_batch(input_sequence_text,
         
         for l in lengths:
         
-            print(f'batching length: {l}')
+            #print(f'batching length: {l}')
             
             num_peptides = num_peptides_per_length[l]
             peptide_list = peptides_with_diff_length[l]
@@ -169,18 +173,18 @@ def split_peptides_with_diff_length_batch(input_sequence_text,
             # max_batch_size x max_batches_per_length, we break up into
             # the max_num_batches, ignoring the max_batch_size
             if (num_batches > max_batches_per_length):
-                print(f'number of batches ({num_batches}) exceeds the maximum batches per length ({max_batches_per_length})')
+                #print(f'number of batches ({num_batches}) exceeds the maximum batches per length ({max_batches_per_length})')
                 num_batches = max_batches_per_length
                 batch_size = math.ceil(num_peptides / num_batches)
             
-            print(f'num batches: {num_batches}')
-            print(f'batch size: {batch_size}')
+            #print(f'num batches: {num_batches}')
+            #print(f'batch size: {batch_size}')
             index_start = 0
             for b_num in range(0, num_batches):
                 # find the index end
                 index_end = min(index_start + batch_size, len(peptide_list))        
-                print(f'index start: {index_start}')
-                print(f'index end: {index_end}')
+                #print(f'index start: {index_start}')
+                #print(f'index end: {index_end}')
                 #print(peptide_list[index_start:index_end])
                 batched_peptide_list.append(peptide_list[index_start:index_end])
                 # increment index_start
@@ -226,36 +230,35 @@ def split_parameters(input_data, split_inputs_dir=None):
     peptide_file_path_length_pairs = []
     input_sequence_text = input_data.pop('input_sequence_text','')
     peptide_length_range = input_data.get('peptide_length_range', None)
+    # peptide_shift is a synonym of peptide_spacing
+    peptide_shift = int(input_data.get('peptide_shift', 0))
+    if peptide_shift:
+        peptide_spacing = peptide_shift
+    else:
+        peptide_spacing = int(input_data.get('peptide_spacing', 1))
     peptide_list = input_data.pop('peptide_list', '')
     input_sequence_fasta_uri = input_data.pop('input_sequence_fasta_uri', '')
     alleles = input_data.get('alleles').split(',')
-    # remove empty alleles
-    alleles = [allele.strip() for allele in alleles if allele.strip()]
-    # remove repeated alleles but keep the original order
-    # and print a warning if there are repeated alleles
-    if len(alleles) != len(set(alleles)):
-        print('warning: repeated alleles found, removing duplicates')
-        alleles = list(dict.fromkeys(alleles))
     predictors = input_data.get('predictors')
-    allele_validator = Allele_Validator()
+    allele_validator = AlleleValidator()
     if input_sequence_fasta_uri:
         input_sequence_text_file_path = save_file_from_URI(input_sequence_fasta_uri, target_dir=split_inputs_dir)
         to_delete.append(input_sequence_text_file_path)
         with open(input_sequence_text_file_path, 'r') as r_file:
             input_sequence_text = r_file.read()
         # must have peptide_length_ranged
-        peptide_lists_with_diff_len = split_peptides_with_diff_length_batch(input_sequence_text, peptide_length_range)
+        peptide_lists_with_diff_len = split_peptides_with_diff_length_batch(input_sequence_text, peptide_length_range, peptide_spacing)
 
-        sequence_peptide_index = get_peptide_list_with_start_and_stop_and_sequencenumber(input_sequence_text, alleles, peptide_length_range)
+        sequence_peptide_index = get_peptide_list_with_start_and_stop_and_sequencenumber(input_sequence_text, alleles, peptide_length_range, peptide_spacing)
     elif input_sequence_text:
         with tempfile.NamedTemporaryFile(mode='w', dir=split_inputs_dir, delete=False) as tmp_peptides_file:
             input_sequence_text_file_path = tmp_peptides_file.name
             to_delete.append(input_sequence_text_file_path)
             tmp_peptides_file.write(input_sequence_text)
         # must have peptide_length_ranged
-        peptide_lists_with_diff_len = split_peptides_with_diff_length_batch(input_sequence_text, peptide_length_range)
+        peptide_lists_with_diff_len = split_peptides_with_diff_length_batch(input_sequence_text, peptide_length_range, peptide_spacing)
 
-        sequence_peptide_index = get_peptide_list_with_start_and_stop_and_sequencenumber(input_sequence_text, alleles, peptide_length_range)
+        sequence_peptide_index = get_peptide_list_with_start_and_stop_and_sequencenumber(input_sequence_text, alleles, peptide_length_range, peptide_spacing)
 
     elif peptide_list:
         peptide_lists_with_diff_len = get_peptides_with_diff_length(peptide_list)
@@ -292,39 +295,18 @@ def split_parameters(input_data, split_inputs_dir=None):
     if basic_predictors:
         binding_method = basic_predictors[0]['mhc_binding_method']
         predictors = [p for p in predictors if p['type']!='binding' or p['method']!=binding_method]
-    has_non_allele_immunogenicity = False
+    has_immunogenicity_splited = False
     for allele in alleles:
+        alle = allele_validator.convert_synonym_to_iedblabel(allele)
+        if not alle:
+            alle = allele
         for predictor in predictors:
             predictor_type = predictor.get('type','')
-            # only split when mask_choice == 'by_allele'
             if predictor_type == 'immunogenicity':
-                if predictor.get('mask_choice','') == 'by_allele':
-                    # for immunogenicity: skip invalid alleles if mask_choice == by_allele
-                    if not allele_validator.validate_alleles(allele, 'immunogenicity'):
-                        print("Warning: allele {} is not available for immunogenicity.".format(allele))
-                        continue
-                    methods = (predictor.get('method','immunogenicity'),)
-                    for method in methods:
-                        data_unit = input_data.copy()
-                        unit_predictor = predictor.copy()
-                        unit_predictor['method'] = method
-                        data_unit['alleles'] = allele
-                        data_unit['predictors'] = [unit_predictor]
-                        output_data.append(data_unit)
+                if has_immunogenicity_splited:
+                    continue
                 else:
-                    # only create one job for all alleles, do not join alleles
-                    if has_non_allele_immunogenicity:
-                        continue
-                    has_non_allele_immunogenicity = True
-                    methods = (predictor.get('method','immunogenicity'),)
-                    for method in methods:
-                        data_unit = input_data.copy()
-                        unit_predictor = predictor.copy()
-                        unit_predictor['method'] = method
-                        data_unit['predictors'] = [unit_predictor]
-                        output_data.append(data_unit)
-                continue
-
+                    has_immunogenicity_splited = True
             method = predictor.get('method','')
             if not method:
                 method = predictor_type
@@ -332,35 +314,35 @@ def split_parameters(input_data, split_inputs_dir=None):
             method = method.replace('recommended_epitope','netmhcpan_el').replace('recommended_binding','netmhcpan_ba')
             # for consensus
             if method == 'consensus':
-                methods = MHCI_CONSENSUS_METHOD_SET
+                methods = MHCII_CONSENSUS_METHOD_SET
             else:
-                if has_consensus and method in MHCI_CONSENSUS_METHOD_SET:
+                if has_consensus and method in MHCII_CONSENSUS_METHOD_SET:
                     continue
                 else:
                     methods = (method,)
             for method in methods:
-                if method and method not in [ 'netchop', 'immunogenicity']:
+                if method and method not in [ 'netchop', 'immunogenicity', 'cd4episcore']:
                     if method == 'basic_processing':
                         # TODO: run validation for its mhc_binding_method for basic_processing
                         binding_method = predictor['mhc_binding_method']
-                        if not allele_validator.validate_alleles(allele, binding_method):
-                            print('warning: invalid allele %s for binding_method %s' % (allele, binding_method))
+                        if not allele_validator.validate_alleles(alle, binding_method):
+                            print('warning: invalid allele %s for binding_method %s' % (alle, binding_method))
                             continue
-                    elif not allele_validator.validate_alleles(allele, method):
+                    elif not allele_validator.validate_alleles(alle, method):
                         # TODO: add warninigs e.g. allele Mamu-A2*05:10 is not available for method smmpmbec
-                        print('warning: invalid allele %s for method %s' % (allele, method))
+                        print('warning: invalid allele %s for method %s' % (alle, method))
                         continue
                 data_unit = input_data.copy()
                 unit_predictor = predictor.copy()
                 unit_predictor['method'] = method
-                data_unit['alleles'] = allele
+                data_unit['alleles'] = alle
                 data_unit['predictors'] = [unit_predictor]
                 output_data.append(data_unit)
                 # for basic processing, add the binding predictor it require
                 if method == 'basic_processing':
                     binding_method = unit_predictor['mhc_binding_method'].replace('recommended_epitope','netmhcpan_el').replace('recommended_binding','netmhcpan_ba')
                     data_unit = input_data.copy()
-                    data_unit['alleles'] = allele
+                    data_unit['alleles'] = alle
                     data_unit['predictors'] = [{"type": "binding", "method": binding_method}]
                     output_data.insert(-1, data_unit)
 
@@ -385,6 +367,7 @@ def split_parameters(input_data, split_inputs_dir=None):
                 new_data_unit = data_unit.copy()
                 new_data_unit['peptide_file_path'] = peptide_file_path
                 new_data_unit['peptide_length_range'] = [peptide_length, peptide_length]
+                new_data_unit['input_sequence_text'] = input_sequence_text
                 new_output_data.append(new_data_unit)
         # to add predictors for netchop/netctl/netctlpan here
         # only peptide_list will not work for netchop/netctl/netctlpan
@@ -409,6 +392,7 @@ def split_parameters(input_data, split_inputs_dir=None):
                         new_data_unit = data_unit.copy()
                         new_data_unit['input_sequence_text_file_path'] = input_sequence_text_file_path
                         new_output_data.append(new_data_unit)
+            
         output_data = new_output_data
     return output_data, sequence_peptide_index, has_consensus
 
@@ -441,7 +425,7 @@ def split_parameters_file(json_filename, parameters_output_dir=None, split_input
     base_dir = os.path.abspath(os.path.join(parameters_output_dir, os.pardir))
     job_descriptions_path = os.path.abspath(os.path.join(base_dir, 'job_descriptions.json'))
     sequence_peptide_index_path = os.path.abspath(os.path.join(result_output_dir, 'sequence_peptide_index.json'))
-    mhci_predict_executable_path = os.path.abspath(os.path.join(os.path.dirname(__file__), 'tcell_mhci.py'))
+    mhcii_predict_executable_path = os.path.abspath(os.path.join(os.path.dirname(__file__), 'tcell_mhcii.py'))
 
     os.makedirs(result_output_dir, exist_ok=True)
     job_descriptions = []
@@ -451,7 +435,7 @@ def split_parameters_file(json_filename, parameters_output_dir=None, split_input
         job_id = i
         data_unit_file_path = os.path.join(parameters_output_dir, '%d.json' % i)
         save_json(data_unit, data_unit_file_path)
-        shell_cmd='%s -j %s/%s.json -o %s/%s -f json' % (mhci_predict_executable_path, parameters_output_dir, job_id, result_output_dir, job_id)
+        shell_cmd='%s -j %s/%s.json -o %s/%s -f json' % (mhcii_predict_executable_path, parameters_output_dir, job_id, result_output_dir, job_id)
         if assume_valid:
             shell_cmd += ' --assume-valid'
         job_description = dict(
@@ -469,7 +453,7 @@ def split_parameters_file(json_filename, parameters_output_dir=None, split_input
     # job_id == -1 means no job is required to run
     if job_id > -1:
         job_id +=1
-        shell_cmd='%s --aggregate --job-desc-file=%s --aggregate-input-dir=%s --aggregate-result-dir=%s' % (mhci_predict_executable_path, job_descriptions_path, result_output_dir, aggregate_dir)
+        shell_cmd='%s --aggregate --job-desc-file=%s --aggregate-input-dir=%s --aggregate-result-dir=%s' % (mhcii_predict_executable_path, job_descriptions_path, result_output_dir, aggregate_dir)
         if has_consensus:
             shell_cmd += " --has-consensus"
         if keep_empty_row:
@@ -509,4 +493,4 @@ if __name__ == '__main__':
     
     # summarize the list here
     #print(sp)
-
+    
