@@ -85,20 +85,22 @@ class Prediction:
         version_info = (
             "tool_group\tmethod\tversion\n"
             "-------\t-------\t-------\n"
-            "binding\tconsensus\t2.18\n"
-            "binding\tann\t4.0\n"
-            "binding\tnetmhcpan_ba\t4.1\n"
-            "binding\tnetmhcpan_el\t4.1\n"
-            "binding\tcomblib_sidney2008\t1.0\n"
-            "binding\tsmm\t1.0\n"
-            "binding\tsmmpmbec\t1.0\n"
-            "binding\tmhcnp\t2.0.3\n"
-            "binding\tmhcflurry\t2.1.1\n"
-            "processing\tbasic_processing\t1.0\n"
-            "processing\tnetchop\t3.1\n"
-            "processing\tnetctl\t1.1\n"
-            "processing\tnetctlpan\t1.1\n"
-            "immunogenicity\timmunogenicity\t1.0\n"
+            "binding\tNetMHCIIpan EL\t4.1\n"
+            "binding\tNetMHCIIpan BA\t4.1\n"
+            "binding\tNetMHCIIpan EL\t4.3\n"
+            "binding\tNetMHCIIpan BA\t4.3\n"
+            "binding\tNetMHCIIpan EL\t4.2\n"
+            "binding\tNetMHCIIpan BA\t4.2\n"
+            "binding\tNetMHCIIpan EL\t4.0\n"
+            "binding\tNetMHCIIpan BA\t4.0\n"
+            "binding\tNetMHCIIpan\t3.2\n"
+            "binding\tNetMHCIIpan\t3.1\n"
+            "binding\tSMM_align (NetMHCII) *\t1.1\n"
+            "binding\tNN_align (NetMHCII) *\t2.2\n"
+            "binding\tNN_align (NetMHCII) *\t2.3\n"
+            "binding\tCombinatorial library\t1.0\n"
+            "binding\tSturniolo\t1.0\n"
+            "Immunogencity\CD4Episcore\t1.0\n"
         )
         print(version_info)
 
@@ -273,7 +275,6 @@ length: a length"
  
         except Exception as e:
             eprint(str(e))
-            raise
             exit(1)
 
 def run_batch_predictions(options, args):
